#!/usr/bin/env python
#
# Copyright 2012 Shadow Robot Company Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import division
import os
import sys

import roslib
roslib.load_manifest('sr_gui_joint_slider')
import rospy

from xml.etree import ElementTree as ET

from rosgui.QtBindingHelper import loadUi
from QtCore import QEvent, QObject, Qt, QTimer, Slot
from QtGui import QDockWidget, QShortcut, QMessageBox, QFrame
from pr2_mechanism_msgs.srv import ListControllers, SwitchController, LoadController
from jointSlider import JointController, Joint, CANHandSlider, EtherCATHandSlider, ArmSlider, CANHandSelectionSlider, EtherCATSelectionSlider, ArmSelectionSlider
from etherCAT_hand_lib import EtherCAT_Hand_Lib
from shadowhand_ros import ShadowHand_ROS

class SrGuiJointSlider(QObject):

    def __init__(self, parent, plugin_context):
        super(SrGuiJointSlider, self).__init__(parent)
        self.setObjectName('SrGuiJointSlider')

        main_window = plugin_context.main_window()
        self._widget = QDockWidget(main_window)

        ui_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../uis/SrJointSlider.ui')
        loadUi(ui_file, self._widget)
        
        self._widget.setObjectName('SrJointSliderUi')
        if plugin_context.serial_number() > 1:
            self._widget.setWindowTitle(self._widget.windowTitle() + (' (%d)' % plugin_context.serial_number()))
        main_window.addDockWidget(Qt.RightDockWidgetArea, self._widget)
        
        # trigger deleteLater for plugin when _widget is closed
        self._widget.installEventFilter(self)
        
        #read the xml configuration file
        config_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../model/slide_joints.xml')
        self.tree = ET.ElementTree()
        self.tree.parse(config_file)
        self.robots = None
        
        self.sliders = list()
        self.selection_slider = None
        
        robot_types = self.get_robot_types(self.tree)
        self._widget.comboBox.addItems(robot_types)
        self._widget.comboBox.setCurrentIndex(-1)
        
        self.is_active = True
        self.robot_lib_CAN = None
        self.robot_lib_eth = None
        
        self._widget.comboBox.activated.connect(self.on_robot_type_changed_)

    @Slot(str)
    def _unregister(self):         
        if self.robot_lib_eth is not None:    
            self.robot_lib_eth.on_close()

    def eventFilter(self, obj, event):
        if obj is self._widget and event.type() == QEvent.Close:
            # TODO: ignore() should not be necessary when returning True
            event.ignore()
            self.deleteLater()
            return True
        return QObject.eventFilter(self, obj, event)

    def close_plugin(self):
        self._unregister()
        self._widget.close()
        self._widget.deleteLater()

    def save_settings(self, global_settings, perspective_settings):
        print "saving settings"

    def restore_settings(self, global_settings, perspective_settings):
        print "restoring settings"
        
    def get_robot_types(self, config):
        if sys.version_info < (2, 7):
            self.robots = list(config.getiterator("robot"))
        else:
            self.robots = list(config.iter("robot"))
        robot_types = list()
        for element in self.robots:
            robot_types.append(element.attrib["name"])
        return robot_types
    
    def on_robot_type_changed_(self):
        #We first read the config from the file into a joints list
        j = self.robots[self._widget.comboBox.currentIndex()].find("joints")
        if sys.version_info < (2, 7):
            config_joints = list(j.getiterator("joint"))
        else:
            #For Python version 2.7 and later
            config_joints = list(j.iter("joint"))
        
        #Read the joints configuration
        self.joints = list()
        for config_joint in config_joints:
            if sys.version_info < (2, 7):
                config_joint_controllers = list(config_joint.getiterator("controller"))
            else:
                config_joint_controllers = list(config_joint.iter("controller"))
            joint_controllers = list()
            for config_controller in config_joint_controllers:
                name = config_controller.attrib["name"]
                command_topic = config_controller.findtext("command_topic")
                msg_type = config_controller.findtext("msg_type")
                min = int(config_controller.findtext("min"))
                max = int(config_controller.findtext("max"))
                controller = JointController(name, command_topic, msg_type, min, max)
                joint_controllers.append(controller)
            name = config_joint.attrib["name"]
            joint = Joint(name, joint_controllers)
            self.joints.append(joint)     
        
        #Clear existing slider widgets from layout
        self.delete_old_sliders_()
        
        #Load the correct robot library
        self.load_robot_library_()
        
        if self.is_active:            
            #Create and load the new slider widgets
            self.load_new_sliders_()

    def delete_old_sliders_(self):
        #Clear existing slider widgets from layout
        for old_slider in self.sliders:
            self._widget.horizontalLayout.removeWidget(old_slider)
            old_slider.close()
            old_slider.deleteLater()
        
        #Empty the slider list
        self.sliders = list()
            
        if(self.selection_slider is not None):
            self._widget.horizontalLayout.removeWidget(self.selection_slider)
            self.selection_slider.close()
            self.selection_slider.deleteLater()
            self.selection_slider = None
            
    def load_robot_library_(self):
        #Load the correct robot library
        if self._widget.comboBox.currentText() in ["CAN Hand", "Arm"]:
            self.is_active = True
            if self.robot_lib_CAN is None:
                self.robot_lib_CAN = ShadowHand_ROS()
        elif self._widget.comboBox.currentText() == "EtherCAT Hand":
            self.is_active = True
            if self.robot_lib_eth is None:
                self.robot_lib_eth = EtherCAT_Hand_Lib()
                if not self.robot_lib_eth.activate_joint_states():
                    btn_pressed = QMessageBox.warning(self._widget, "Warning", "The EtherCAT Hand node doesn't seem to be running. Do you still want to continue? The hand states will not be updated.", buttons = QMessageBox.Ok |  QMessageBox.Cancel)
                    if btn_pressed == QMessageBox.Cancel:
                        self.is_active = False
                        if self.robot_lib_eth is not None:    
                            self.robot_lib_eth.on_close()
                            self.robot_lib_eth = None
        else:
            rospy.logwarn("Unknown robot name: " + self._widget.comboBox.currentText())
            
    def load_new_sliders_(self):
        #Create the new slider widgets
        self.sliders = list()
        for joint in self.joints:
            if self._widget.comboBox.currentText() == "CAN Hand":
                slider = CANHandSlider(joint, '../uis/Slider.ui', self.robot_lib_CAN, self, self._widget.scrollAreaWidgetContents)
            elif self._widget.comboBox.currentText() == "EtherCAT Hand":
                slider = EtherCATHandSlider(joint, '../uis/Slider.ui', self.robot_lib_eth, self, self._widget.scrollAreaWidgetContents)
            elif self._widget.comboBox.currentText() == "Arm":
                slider = ArmSlider(joint, '../uis/Slider.ui', self.robot_lib_CAN, self, self._widget.scrollAreaWidgetContents)
            else:
                rospy.logwarn("Unknown robot name: " + self._widget.comboBox.currentText())           
            
            slider.setMaximumWidth(100)
            #Load the new slider
            self._widget.horizontalLayout.addWidget(slider)
            #Put the slider in the list         
            self.sliders.append(slider)
        
        #Create the slider to move all the selected joint sliders
        if self._widget.comboBox.currentText() == "CAN Hand":
            self.selection_slider = CANHandSelectionSlider("Change sel.", 0, 100, '../uis/SelectionSlider.ui', self.robot_lib_CAN, self, self._widget.scrollAreaWidgetContents)
        elif self._widget.comboBox.currentText() == "EtherCAT Hand":
            self.selection_slider = EtherCATSelectionSlider("Change sel.", 0, 100, '../uis/SelectionSlider.ui', self, self._widget.scrollAreaWidgetContents)
        elif self._widget.comboBox.currentText() == "Arm":
            self.selection_slider = ArmSelectionSlider("Change sel.", 0, 100, '../uis/SelectionSlider.ui', self.robot_lib_CAN, self, self._widget.scrollAreaWidgetContents)
        else:
            rospy.logwarn("Unknown robot name: " + self._widget.comboBox.currentText())
        
        self.selection_slider.setMaximumWidth(100)
        self._widget.horizontalLayout.addWidget(self.selection_slider)
